## ############################################################################
##
## The 2022 Europe Report of the Lancet Countdown on Health and Climate Change:
##                 towards a climate resilient future
##
## Indicator 1.2.2: Drought Indicator
##
## ############################################################################
##
## Authors: 
## - Balakrishnan Solaraju Murali, ORCID: 0000-0003-0013-3295
## - Nube González Reviriego, ORCID: 0000-0002-5919-6701
##
## Contributor:
## - Prof. Rachel Lowe, ORCID: 0000-0003-3939-7343
##
## Contact information: balakrishnan.solaraju@bsc.es
## 
## ############################################################################

# Script to estimate drought indicator: Standardised Precipitation-Evapotranspiration Index (SPEI)

library(startR)
library(multiApply)
library(SPEI)
library(s2dverification)

################################################################################################
################################## LOAD THE DATA: OBS ##########################################
################################################################################################

variables = c('tasmax','tasmin','prlr')
current_dataset <- 'era5land'
retrieve <- TRUE

years_obs = 1950:2021
n_procs_obs <- 24

# European Domain
lat_min <- 25
lat_max <- 75
lon_min <- -15
lon_max <- 44

# Load monthly tas and transform

variable <- variables[1]
path_obs_tasmax='/era5land/monthly_mean/tasmax_f24h/$var$_$year$$month$.nc'

  data_tasmax_obs = Start(dataset = path_obs_tasmax,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                       lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       num_procs = n_procs_obs, 
                       retrieve = retrieve)

  names(dim(data_tasmax_obs)) <- c("dat","member","sdate","time","latitude","longitude")

  # From K to degree C
  
  factor_tmean=273.15
  data_tasmax_obs <- data_tasmax_obs - factor_tmean
          
# Check the longitude and latitude
lat <- as.vector(attr(data_tasmax_obs, 'Variables')$dat1$lat)
lon <- as.vector(attr(data_tasmax_obs, 'Variables')$dat1$lon)
lat_mask <- InsertDim(InsertDim(lat,1,1),3,length(lon))
names(dim(lat_mask)) <- c("dat","latitude","longitude")

# Load monthly tmin and transform

variable <- variables[2]
path_obs_tasmin='/era5land/monthly_mean/tasmin_f24h/$var$_$year$$month$.nc'

  data_tasmin_obs = Start(dataset = path_obs_tasmin,
                       var = variable,
                       year = paste0(years_obs),
                       month = 'all',
                       lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                       lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                       synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                       return_vars = list(lat = 'dataset', lon = 'dataset'),
                       num_procs = n_procs_obs,
                       retrieve = retrieve)

  names(dim(data_tasmin_obs)) <- c("dat","member","sdate","time","latitude","longitude")

  # From K to degree C

  factor_tmean=273.15
  data_tasmin_obs <- data_tasmin_obs - factor_tmean

# Load monthly  prlr and transform

variable <- variables[3]

path_obs_pre='/era5land/monthly_mean/prlr_f1h/$var$_$year$$month$.nc'

data_pre_obs = Start(dataset = path_obs_pre,
                     var = variable,
                     year = paste0(years_obs),
                     month = 'all',
                     lat = values(list(lat_min,lat_max)), lat_reorder = Sort(decreasing = TRUE), lat_var = 'lat',
                     lon = values(list(lon_min,lon_max)), lon_reorder = CircularSort(-180,180), lon_var = 'lon',
                     synonims = list(lat = c('lat','latitude'), lon = c('lon','longitude')),
                     return_vars = list(lat = 'dataset', lon = 'dataset'),
                     num_procs = n_procs_obs, 
                     retrieve = retrieve)

names(dim(data_pre_obs)) <- c("dat","member","sdate","time","latitude","longitude")

# From m.s-1 to mm.day-1
factor_pre=(60*60*24*1000) 
data_pre_obs <- data_pre_obs * (factor_pre*30.4375)

data_pre_obs[which(is.na(data_pre_obs))] <- 0
data_tasmax_obs[which(is.na(data_tasmax_obs))] <- 0
data_tasmin_obs[which(is.na(data_tasmin_obs))] <- 0

################################################################################################
################################### PET6 estimation  ###########################################
################################################################################################
# Estimating Potential Evapotranspiration (PET)

  library(zoo)
  pet_method_x <- c('thornthwaite','hargreaves')

  n_procs <- 64

  # PET - Hg approach
  pet_method <- pet_method_x[2]
  print(pet_method)
  
  PET_estimated <- Apply(data=list(data_tasmax_obs,data_tasmin_obs,lat_mask), 
                  target_dims = list(data_tasmax_obs=c('time','sdate'),data_tasmin_obs=c('time','sdate'),lat_mask=c('dat')), 
                  pet_method=pet_method, 
                  output_dims = c('time','sdate'),
                  fun=atomic_pet, ncores = n_procs)
  
  data_pet_obs <- drop(PET_estimated$output1)
  
  print('PET estimation completed...')
  
################################################################################################
################################### P - PET estimation  ########################################
################################################################################################
  
  data_pre_obs_x <- aperm(drop(data_pre_obs),c(2,1,3,4))
  
  diff_P_PET <- data_pre_obs_x - data_pet_obs
  rm(data_pre_obs_x)
  
  # Accumulation

  accum_scale <- 6
  accum <- Apply(data=list(diff_P_PET), 
                  target_dims = list(diff_P_PET=c('time','sdate')), 
                  accum_scale=accum_scale,
                  output_dims = c('time','sdate'),
                  fun=atomic_accum, ncores = n_procs)
  
  data_accum_obs <- accum$output1[,2:as.numeric(dim(data_tasmax_obs)['sdate']),,]
  saveRDS(data_accum_obs, file=paste('/rdata/accum6_Hg.Rds', sep=""))
  
  
################################################################################################
############################ SPEI estimation - Grid level ######################################
################################################################################################
  
  # Standardization
  spei6_cc <- Apply(data=list(data_accum_obs), 
                    target_dims = list(data_accum_obs=c('time','sdate')),
                    fun=atomic_spei, ncores = n_procs)
  
  data_spei_obs <- spei6_cc$output1
  
  saveRDS(data_spei_obs, file=paste('/rdata/spei6_Hg.Rds', sep=""))
  print('SPEI6 estimation completed...')
  rm(data_accum_obs,data_spei_obs)
  
################################################################################################
############################ SPEI estimation - NUTS2 level #####################################
################################################################################################
  
### NUTS 2 averaging with accumulated values and SPEI computation ###

data_accum_obs <- readRDS(file=paste('~/rdata/accum6_Hg.Rds', sep=""))

mask <- readRDS(file=paste('~/rdata/NUTS2_mask_europe_era5land.Rds', sep=""))

eur_mask <- mask$mask
eur_mask[eur_mask == 0] <- NA

# Redo dimension to apply mask

spei_obs_masked <- array(NA, dim=c(334,12,71))

for(i in 1:334){
print(i)
euro_mask_temp <- eur_mask[i,,]
euro_mask_temp <- InsertDim(InsertDim(euro_mask_temp,1,12),2,71)

b <- euro_mask_temp * data_accum_obs

c <- s2dverification::Mean1Dim(b,4)
c <- s2dverification::Mean1Dim(c,3)

spei_obs_masked[i,,] <- c

}

data_accum_obs_nuts2 <- aperm(spei_obs_masked, c(2,3,1))
names(dim(data_accum_obs_nuts2)) <- c("time","sdate","country")

saveRDS(data_accum_obs_nuts2, file=paste('~/rdata/accum6_Hg_nuts2.Rds', sep=""))

# Standardization
spei6_cc <- Apply(data=list(data_accum_obs_nuts2),
                    target_dims = list(data_accum_obs=c('time','sdate')),
                    #output_dims = c('sdate','member'), 
                    fun=atomic_spei, ncores = 16)

data_spei_obs_nuts2 <- spei6_cc$output1

saveRDS(data_spei_obs_nuts2, file=paste('~/rdata/spei6_Hg_nuts2.Rds', sep=""))
